/******************************************************************************
 *    KeyService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IKeySrv>::Name = "KeyService";

	const char * 
	ScriptService<IKeySrv>::DocStrings[] = {
		"KeyService()\n\t"
			"Provides an interface for using keys.",
		"TryToUseKey(key,obj,usage) -> Boolean\n\t"
			"Returns True if the key is valid for this object, or if the\n\t"
			"operation was successful.",
	};

	const PyMethodDef 
	ScriptService<IKeySrv>::Methods[] = {
		{	"TryToUseKey",
			reinterpret_cast<PyCFunction>(TryToUseKey),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IKeySrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IKeySrv>::Name),
		sizeof(ScriptServiceObject<IKeySrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IKeySrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IKeySrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	void
	ScriptService<IKeySrv>::InitModule(PyObject *module)
	{
		PyType_Ready(&TypeInfo);
		PyModule_AddIntConstant(module, "kKeyUseDefault", kKeyUseDefault);
		PyModule_AddIntConstant(module, "kKeyUseOpen", kKeyUseOpen);
		PyModule_AddIntConstant(module, "kKeyUseClose", kKeyUseClose);
		PyModule_AddIntConstant(module, "kKeyUseCheck", kKeyUseCheck);
	}

	ScriptServiceObject<IKeySrv> * 
	ScriptService<IKeySrv>::NewObject(PyObject *)
	{
		IKeySrv *iface = static_cast<IKeySrv*>(g_pScriptManager->GetService(IID_IKeyScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IKeySrv> *obj = PyObject_New(ScriptServiceObject<IKeySrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IKeySrv>::TryToUseKey(ScriptServiceObject<IKeySrv> *self, PyObject *args)
	{
		object key, obj;
		eKeyUse usage;
		if (!PyArg_ParseTuple(args, "O&O&i:TryToUseKey", cvtObject, &key, cvtObject, &obj, &usage))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->TryToUseKey(key,obj,usage)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}


} // namespace PyNumbra


