/******************************************************************************
 *    MonologObject.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifdef MONOOBJECT
#include <Python.h>
#include "pydefs.h"
#include <new>
#include <cstring>
//#include <lg/types.h>
//#include <lg/scrservices.h>
//#include <lg/scrmanagers.h>

#include "pyutils.h"
#include "MonologObject.h"
#include "PyNumbraExceptions.h"


namespace PyNumbra
{

	//const cScrStr MonologObject::_Nil = "";

	const char *MonologObject::Name = "MonologOutput";

	const PyMethodDef MonologObject::Methods[] = {
		{ "close", reinterpret_cast<PyCFunction>(Close), METH_NOARGS, NULL },
		{ "flush", reinterpret_cast<PyCFunction>(Flush), METH_NOARGS, NULL },
		{ "write", reinterpret_cast<PyCFunction>(Write), METH_VARARGS, NULL },
		{ NULL, NULL, 0, NULL }
	};

	const PyGetSetDef MonologObject::Members[] = {
		{ "name", 
		  reinterpret_cast<getter>(GetName), 
		  reinterpret_cast<setter>(SetReadOnly), 
		  NULL, NULL },
		{ "closed", 
		  reinterpret_cast<getter>(GetClosed), 
		  reinterpret_cast<setter>(SetReadOnly), 
		  NULL, NULL },
		{ "softspace", 
		  reinterpret_cast<getter>(GetSoftspace), 
		  reinterpret_cast<setter>(SetSoftspace), 
		  NULL, NULL },
		{ NULL, NULL, NULL, NULL, NULL }
	};
	
	PyTypeObject MonologObject::Type = {
		PyObject_HEAD_INIT(NULL) 0,
		const_cast<char*>(Name),
		sizeof(MonologObject), 0,
		reinterpret_cast<destructor>(Dealloc),
		0, 0, 0, 0, 0,
		0, 0, 0, 0, 0,
		reinterpret_cast<reprfunc>(Str),
		//reinterpret_cast<getattrofunc>(GetAttr),
		//reinterpret_cast<setattrofunc>(SetAttr),
		0, 0, 0,
		Py_TPFLAGS_DEFAULT,
		NULL,
		0, 0, 0, 0, 0, 0,
		const_cast<PyMethodDef*>(Methods), 0,
		const_cast<PyGetSetDef*>(Members),
		0,0,0,0,0,
		reinterpret_cast<initproc>(Init),
		0,0,0,0,
		0,0,0,0,0,0
	};

	PyObject * MonologObject::New(void)
	{
		if (!Type.ob_type)
		{
			if (PyType_Ready(&Type) < 0)
				return NULL;
		}
		/*
		IDebugScrSrv *pDSS = static_cast<IDebugScrSrv*>(g_pScriptManager->GetService(IID_IDebugScriptService));
		if (!pDSS)
		{
			// of course, there's likely no way the user will ever see this message
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		*/
		MonologObject* obMono = PyObject_New(MonologObject, &Type);
		if (obMono)
		{
			//obMono->m_pDSS = pDSS;
			DEBUGPRINTF("=== PyMono Active\n");
		}
		else
		{
			//pDSS->Release();
		}
		return static_cast<PyObject*>(obMono);
	}

	int MonologObject::Init(MonologObject *self, PyObject *, PyObject *)
	{
		self->m_bSoftspace = 0;
		self->m_iPos = 0;
		return 0;
	}

	void MonologObject::Dealloc(MonologObject *self)
	{
		//self->m_pDSS->Release();
		self->ob_type->tp_free(static_cast<PyObject*>(self));
	}

	PyObject* MonologObject::Str(MonologObject *self)
	{
		return PyString_FromFormat("<PyNumbra.MonologOutput at %p>", self);
	}

	PyObject* MonologObject::GetName(MonologObject *self, void*)
	{
		return PyString_FromString("<MonologOutput>");
	}

	PyObject* MonologObject::GetClosed(MonologObject *self, void*)
	{
		PyObject *obRet = Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject* MonologObject::GetSoftspace(MonologObject *self, void*)
	{
		PyObject *obRet = (self->m_bSoftspace) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	int MonologObject::SetReadOnly(MonologObject *self, PyObject *, void*)
	{
		PyErr_SetString(PyExc_TypeError, "Read-only attribute");
		return -1;
	}

	int MonologObject::SetSoftspace(MonologObject *self, PyObject *attr, void*)
	{
		self->m_bSoftspace = PyObject_IsTrue(attr);
		return 0;
	}

	/*
	PyObject* MonologObject::GetAttr(MonologObject *self, PyObject *attr)
	{
		PyObject *obRet = NULL;
		PyObject *obName = PyObject_Str(attr);
		if (obName)
		{
			if (!strcmp("name",PyString_AsString(obName)))
			{
				obRet = PyString_FromString("<MonologOutput>");
			}
			else if (!strcmp("closed",PyString_AsString(obName)))
			{
				obRet = Py_False;
				Py_INCREF(obRet);
			}
			else if (!strcmp("softspace",PyString_AsString(obName)))
			{
				obRet = (self->m_bSoftspace) ? Py_True : Py_False;
				Py_INCREF(obRet);
			}
			else
			{
				obRet = PyObject_GenericGetAttr(self, attr);
			}
			Py_DECREF(obName);
		}
		return obRet;
	}

	int MonologObject::SetAttr(MonologObject *self, PyObject *attr, PyObject *arg)
	{
		int iRet = 0;
		PyObject *obName = PyObject_Str(attr);
		if (obName)
		{
			if (!strcmp("name",PyString_AsString(obName)) || !strcmp("closed",PyString_AsString(obName)))
			{
				PyErr_SetString(PyExc_TypeError, "Read-only attribute");
				iRet = -1;
			}
			else if (!strcmp("softspace",PyString_AsString(obName)))
			{
				self->m_bSoftspace = PyObject_IsTrue(arg);
			}
			else
			{
				iRet = PyObject_GenericSetAttr(self, attr, arg);
			}
			Py_DECREF(obName);
		}
		return iRet;
	}
	*/

	PyObject * MonologObject::Close(MonologObject *self)
	{
		return Flush(self);
	}

	PyObject * MonologObject::Flush(MonologObject *self)
	{
		/*
		if (self->m_iPos)
		{
			self->m_chReadBuf[self->m_iPos] = '\0';
			self->m_pDSS->MPrint(self->m_chReadBuf, _Nil, _Nil, _Nil, _Nil, _Nil, _Nil, _Nil);
			self->m_iPos = 0;
		}
		*/
		PyObject *obRet = Py_None;
		Py_INCREF(obRet);
		return obRet;
	}

	// MPrint always adds a newline to the output. A tad unfortunate,
	// but I think Python always writes full lines.
	PyObject * MonologObject::Write(MonologObject *self, PyObject *args)
	{
		const char *str;
		int strlen;

		if (!PyArg_ParseTuple(args, "s#", &str, &strlen))
		{
			return NULL;
		}
		if (strlen)
		{
			if (strlen < 1022)
			{
				for (int pos = 0; pos < strlen; ++pos)
				{
					self->m_chReadBuf[pos] = (str[pos]) ? : ' ';
				}
				self->m_chReadBuf[strlen] = '\0';
				g_pfnMPrintf("%s", self->m_chReadBuf);
				//self->m_pDSS->MPrint(self->m_chReadBuf, _Nil, _Nil, _Nil, _Nil, _Nil, _Nil, _Nil);
			}
			else
			{
				int strpos = 0;
				do
				{
					int bufpos = 0;
					for (; bufpos < 1022 && strpos < strlen; ++bufpos, ++strpos)
					{
						self->m_chReadBuf[bufpos] = (str[strpos]) ? : ' ';
					}
					self->m_chReadBuf[bufpos] = '\0';
					g_pfnMPrintf("%s", self->m_chReadBuf);
				} while (strpos < strlen);
				/* The old way, dealing with auto-newlines
				for (int strpos = 0; strpos < strlen; )
				{
					int bufpos = 0;
					for (; bufpos < 1022 && strpos < strlen; ++bufpos, ++strpos)
					{
						if ((self->m_chReadBuf[bufpos] = (str[strpos]) ? : ' ') == '\n')
						{
							++strpos;
							break;
						}
					}
					self->m_chReadBuf[bufpos] = '\0';
					g_pfnMPrintf("%s", self->m_chReadBuf);
					//self->m_pDSS->MPrint(self->m_chReadBuf, _Nil, _Nil, _Nil, _Nil, _Nil, _Nil, _Nil);
				}
				*/
			}
		}

		PyObject *obRet = Py_None;
		Py_INCREF(obRet);
		return obRet;
	}

}
#endif
