#ifdef MONOTHREAD
#include <cstring>
#include <cstdio>
#include <io.h>
#include <process.h>
#include <lg/scrservices.h>
#include <lg/scrmanagers.h>

#include "MonologOutput.h"

extern IScriptMan *g_pScriptManager;
extern HANDLE g_hMonoThread;
//extern HANDLE g_hMonoSignaller;

namespace PyNumbra
{

	MonologOutput::~MonologOutput()
	{
		m_pDSS->Release();
		fclose(m_hPipeOut);
	}

	MonologOutput::MonologOutput(int fd)
	{
		m_hPipeOut = fdopen(fd, "r");
		if (!m_hPipeOut)
			throw reinterpret_cast<void*>(NULL);

		m_pDSS = static_cast<IDebugScrSrv*>(g_pScriptManager->GetService(IID_IDebugScriptService));
		if (!m_pDSS)
			throw reinterpret_cast<void*>(NULL);
	}

	unsigned int MonologOutput::OutputThread(MonologOutput *pMono)
	{

		unsigned int pos = 0;
		char c;

		while ((c = fgetc(pMono->m_hPipeOut)) != EOF)
		{
			pMono->m_chReadBuf[pos] = c;
			if (c == '\n' || ++pos == 127)
			{
				pMono->m_chReadBuf[pos] = '\0';
				pMono->m_pDSS->MPrint(pMono->m_chReadBuf, "", "", "", "", "", "", "");
				pos = 0;
			}
		}

		delete pMono;
		_endthreadex(0);
		return 0;
	}

	FILE * OpenMonolog(void)
	{
		int fdPipe[2];
		FILE *hPipeIn;

		if (_pipe(fdPipe, 2048, 0))
			return NULL;
		hPipeIn = fdopen(fdPipe[1], "w");
		if (!hPipeIn)
		{
			close(fdPipe[1]);
			close(fdPipe[0]);
			return NULL;
		}
		MonologOutput *pMono = NULL;
		try
		{
			pMono = new MonologOutput(fdPipe[0]);
		}
		catch (...)
		{
			close(fdPipe[1]);
			close(fdPipe[0]);
			return NULL;
		}

		unsigned int dwThreadId = 0;
		g_hMonoThread = reinterpret_cast<HANDLE>(_beginthreadex(NULL, 0,
						reinterpret_cast<unsigned (__stdcall *)(void*)>(&MonologOutput::OutputThread),
						reinterpret_cast<LPVOID>(pMono), CREATE_SUSPENDED, &dwThreadId));
		if (!g_hMonoThread)
		{
			delete pMono;
			close(fdPipe[1]);
			return NULL;
		}
		
		SetThreadPriority(g_hMonoThread, THREAD_PRIORITY_BELOW_NORMAL);
		ResumeThread(g_hMonoThread);

		return hPipeIn;
	}

	int CloseMonolog(FILE *fp)
	{
		int iRet = fclose(fp);
		if (g_hMonoThread)
		{
			DWORD dwResult = WaitForSingleObject(g_hMonoThread, 5000);
			if (dwResult != WAIT_OBJECT_0)
				TerminateThread(g_hMonoThread, 2);
			
			//CloseHandle(g_hMonoThread);
			g_hMonoThread = NULL;
		}

		return iRet;
	}

}
#endif
