/******************************************************************************
 *    PlayerLimbsService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IPlayerLimbsSrv>::Name = "PlayerLimbsService";

	const char * 
	ScriptService<IPlayerLimbsSrv>::DocStrings[] = {
		"PlayerLimbsService()\n\t"
			"Provides an interface for managing the disembodied arm that appears\n\t"
			"in the player's view.",
		"Equip(obj) -> Integer\n\t"
			"Display the player arm holding the obj.",
		"UnEquip(obj) -> Integer\n\t"
			"Hide the player arm.",
		"StartUse(obj) -> Integer",
		"FinishUse(obj) -> Integer",
	};

	const PyMethodDef 
	ScriptService<IPlayerLimbsSrv>::Methods[] = {
		{	"Equip",
			reinterpret_cast<PyCFunction>(Equip),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"UnEquip",
			reinterpret_cast<PyCFunction>(UnEquip),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"StartUse",
			reinterpret_cast<PyCFunction>(StartUse),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"FinishUse",
			reinterpret_cast<PyCFunction>(FinishUse),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IPlayerLimbsSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IPlayerLimbsSrv>::Name),
		sizeof(ScriptServiceObject<IPlayerLimbsSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IPlayerLimbsSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IPlayerLimbsSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IPlayerLimbsSrv> * 
	ScriptService<IPlayerLimbsSrv>::NewObject(PyObject *)
	{
		IPlayerLimbsSrv *iface = static_cast<IPlayerLimbsSrv*>(g_pScriptManager->GetService(IID_IPlayerLimbsScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IPlayerLimbsSrv> *obj = PyObject_New(ScriptServiceObject<IPlayerLimbsSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IPlayerLimbsSrv>::Equip(ScriptServiceObject<IPlayerLimbsSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Equip", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Equip(obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IPlayerLimbsSrv>::UnEquip(ScriptServiceObject<IPlayerLimbsSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:UnEquip", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->UnEquip(obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IPlayerLimbsSrv>::StartUse(ScriptServiceObject<IPlayerLimbsSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:StartUse", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->StartUse(obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IPlayerLimbsSrv>::FinishUse(ScriptServiceObject<IPlayerLimbsSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:FinishUse", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->FinishUse(obj);
		return PyInt_FromLong(ret);
	}


} // namespace PyNumbra


