/******************************************************************************
 *    PyNumbra.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <exception>
#include <lg/scrmanagers.h>
#include <lg/properties.h>

extern "C" __declspec(dllimport) const char* Py_GetVersion(void);

#include <hacks/dromedhack.h>
#include <hacks/allhacks.h>
#include <hacks/darkprocs.h>

#include "PyNumbra.h"
#include "ScriptInterpreter.h"

using namespace PyNumbra;

extern "C" 
static void InitDromedCommands(void);

static CommandInfo pynumbra_cmds[] = {
	{ "py_refresh_path", 
	  kCmdTypeFunc,
	  reinterpret_cast<cmd_func>(cmd_refreshpath), 
	  "Refresh the list of available Python scripts.",
	  -1
	},
	{ "py_reload_scripts",
	  kCmdTypeIntFunc,
	  reinterpret_cast<cmd_func>(cmd_reloadscripts),
	  "Forces all Python scripts to be reloaded.",
	  -1
	},
	{ "py_eval",
	  kCmdTypeStringFunc,
	  reinterpret_cast<cmd_func>(cmd_evalstring),
	  "Evaluate a Python statement. Print the result to status and mono.",
	  -1
	},
};

extern const char *g_pszVersionString;

char g_szModuleFileName[MAX_PATH];

IMalloc *g_pMalloc = NULL;
IScriptMan *g_pScriptManager = NULL;
CScriptInterpreter *g_pScriptInterpreter = NULL;

mprintf g_pfnMPrintf = NULL;
StatusField g_pfnStatus = NULL;


__declspec(dllexport) BOOL APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		GetModuleFileName(hInstance, g_szModuleFileName, MAX_PATH);
		return TRUE;
	}
	else 
	if (dwReason == DLL_PROCESS_DETACH)
	{
		if (g_pScriptInterpreter)
			delete g_pScriptInterpreter;


		//if (g_pScriptManager)
		//	g_pScriptManager->Release();
		if (g_pMalloc)
			g_pMalloc->Release();
	}
	return TRUE;
}


__declspec(dllexport)  PyNumbra::IScriptInterpreter * __stdcall
PyNumbraInit(const char *pszPythonHome, IScriptMan *pScriptMan, IMalloc *pMalloc, mprintf pfnMPrintf)
{
	if (!pScriptMan || !pMalloc)
		return NULL;

	if (!g_pMalloc)
	{
		g_pMalloc = pMalloc;
		g_pMalloc->AddRef();
	}
	if (!g_pScriptManager)
	{
		g_pScriptManager = pScriptMan;
		g_pScriptManager->AddRef();
	}

	g_pfnMPrintf = pfnMPrintf;

	try
	{
		if (!g_pScriptInterpreter)
		{
			g_pScriptInterpreter = new CScriptInterpreter();
			if (pszPythonHome)
				g_pScriptInterpreter->SetPythonHome(pszPythonHome);
			// cheap hacks and candy
			InitDromedCommands();
			g_pfnMPrintf("=== PyNumbra imbedded Module for Python, Copyright (C) 2004 Tom N Harris\n");
			g_pfnMPrintf("=== version %s\n", g_pszVersionString);
			g_pfnMPrintf("=== Python version %s\n", Py_GetVersion());
			g_pfnMPrintf("=== PyNumbra comes with ABSOLUTELY NO WARRANTY. This is free software and you are welcome to\n"
                         "=== redistribute it under certain conditions. See the GNU General Public License for details.\n\n");
		}
		g_pScriptInterpreter->AddRef();
	}
	catch (std::exception &err)
	{
		g_pfnMPrintf("!!! Could not initialize PyNumbra: %s\n\n", err.what());
		g_pMalloc->Release();
		g_pScriptManager->Release();
		return NULL;
	}

	return static_cast<IScriptInterpreter*>(g_pScriptInterpreter);
}

// This doesn't work.
// Dromed squashes case before processing the command,
// which screws up most of the PyNumbra keywords.
int __cdecl cmd_evalstring(const char *arg)
{
	if (!g_pScriptInterpreter)
		return 0;
	const char *cmd = arg;
	while (*cmd == ' ') ++cmd;
	return g_pScriptInterpreter->Eval(cmd);
}

int __cdecl cmd_refreshpath(void)
{
	if (g_pScriptInterpreter)
		try
			{ g_pScriptInterpreter->GetScriptNames(); }
		catch (...)
			{ return 0; }
	return 1;
}

int __cdecl
cmd_reloadscripts(int iObj)
{
	if (!g_pScriptInterpreter)
		return 0;

	IGenericProperty *pScrProp;
	{
		IPropertyManager *pPropMan;
		if (E_NOINTERFACE == g_pScriptManager->QueryInterface(IID_IPropertyManager, reinterpret_cast<void**>(&pPropMan)))
			return 0;
		IProperty *pProp = pPropMan->GetPropertyNamed("Scripts");
		pPropMan->Release();
		if (!pProp)
			return 0;
		if (E_NOINTERFACE == pProp->QueryInterface(IID_IScriptProperty, reinterpret_cast<void**>(&pScrProp)))
		{
			pProp->Release();
			return 0;
		}
		pProp->Release();
	}
	
	if (iObj)
		g_pScriptManager->ForgetObj(iObj);
	else
		g_pScriptManager->ForgetAllObjs();

	g_pScriptInterpreter->InvalidateScriptCache();
	if (iObj == 0)
	{
		sPropertyObjIter propiter;
		int id;
		pScrProp->IterStart(&propiter);
		while (pScrProp->IterNext(&propiter, &id))
			pScrProp->Touch(id);
		pScrProp->IterStop(&propiter);
	}
	else
		pScrProp->Touch(iObj);
	pScrProp->Release();

	return 1;
}

void InitDromedCommands(void)
{
	HMODULE hHackModule = LoadLibraryA("dromedhack.dll");
	if (!hHackModule)
		return;

	IsEditor pfnIsEditor = reinterpret_cast<IsEditor>(GetProcAddress(hHackModule, "_IsEditor@0"));
	if (!pfnIsEditor || !pfnIsEditor())
	{
		FreeLibrary(hHackModule);
		return;
	}

	GetDromedProcAddress pfnGetDromedProcAddress = reinterpret_cast<GetDromedProcAddress>(GetProcAddress(hHackModule, "_GetDromedProcAddress@4"));
	if (pfnGetDromedProcAddress)
	{
		CommandRegister pfnCommandRegister = reinterpret_cast<CommandRegister>(pfnGetDromedProcAddress(__COMMANDREGISTER));
		if (pfnCommandRegister)
		{
#ifdef DEBUG_VERBOSE
		g_pfnMPrintf("=== Setting-up Dromed commands.\n");
#endif
			pfnCommandRegister(pynumbra_cmds, sizeof(pynumbra_cmds)/sizeof(pynumbra_cmds[0]), -1);
			// Okay, so we've just installed some callbacks for Dromed pointing to 
			// code in this, a DLL. There's no way to tell Dromed when the module is 
			// unloaded, so if it tries to call one of our commands, that would be a 
			// Bad Thing(tm). Thus, we want to increment the reference count of the 
			// module to prevent it from being unloaded.
			LoadLibrary(g_szModuleFileName);
		}

		g_pfnStatus = reinterpret_cast<StatusField>(pfnGetDromedProcAddress(__STATUSFIELD));
	}
	FreeLibrary(hHackModule);
}

