/******************************************************************************
 *    PyNumbra.h
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#ifndef PYNUMBRA_H
#define PYNUMBRA_H

#include <windows.h>
#include <lg/types.h>
#include <lg/scrmanagers.h>

#ifndef Py_PYTHON_H
// To limit the complexity and namespace pollution,
// we don't include Python.h here. Clients of the
// interpreter should treat PyObject as an opaque type.
struct _object;
typedef struct _object PyObject;
#endif

namespace PyNumbra
{
	class IScriptInterpreter : public IUnknown
	{
	public:
		virtual void __stdcall 
		SetPythonHome(const char *pszPath) = 0;

		virtual int __stdcall
		SetScriptPath(const char *pszPath) = 0;

		// Reads the list of script names from the search path
		virtual const char * const * __stdcall
		GetScriptNames(void) = 0;

		// Forgets the script objects that have already been parsed.
		// It's up to you to figure out how to tell the currently running 
		// scripts to restart.
		virtual void __stdcall 
		InvalidateScriptCache(void) = 0;

		// Returns the class pointer or NULL
		virtual PyObject * __stdcall
		LoadScript(const char *pszName) = 0;

		virtual void __stdcall
		UnloadScript(const char *pszName, PyObject *pClass) = 0;

		// Returns a script instance pointer or NULL
		virtual PyObject * __stdcall
		BeginScript(const char *pszName, PyObject *pClass, int iObjId) = 0;

		virtual void __stdcall
		EndScript(PyObject *pScript, PyObject *pClass, int iObjId) = 0;

		virtual int __stdcall
		ReceiveMessage(PyObject *pScript, const sScrMsg *pMsg, sMultiParm *pReply) = 0;
	
		virtual int __stdcall 
		Eval(const char *pszCmd) = 0;
	};
}

extern "C"
{
	__declspec(dllexport) BOOL APIENTRY
	DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID);

	__declspec(dllexport)  PyNumbra::IScriptInterpreter * __stdcall
	PyNumbraInit(const char *pszPythonHome, IScriptMan *pScriptMan, IMalloc *pMalloc, int (__cdecl *pfnMPrintf)(const char*,...));

	typedef PyNumbra::IScriptInterpreter*(__stdcall *PyNumbraInitProc)(char*,IScriptMan*,IMalloc*,int (__cdecl *)(const char*,...));

	int __cdecl cmd_refreshpath(void);
	int __cdecl cmd_reloadscripts(int);
	int __cdecl cmd_evalstring(const char*);

} // extern "C"

#endif
