/******************************************************************************
 *    PyScript.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <new>

#include "PyScript.h"
#include "ScriptModule.h"


cPyScript::~cPyScript()
{
	if (m_pClass)
	{
#ifdef DEBUG_VERBOSE
		g_pfnMPrintf("--- Removing PyScript %s [%p] from #%d\n", m_szName, m_pClass, m_iObjId);
#endif
		g_pScriptInterpreter->UnloadScript(m_szName, m_pClass);
	}
}

cPyScript::cPyScript(const char *pszName, int iHostObjId)
	: cScript(pszName, iHostObjId), m_pClass(NULL), m_pScript(NULL)
{
	if (m_szName)
	{
		m_pClass = g_pScriptInterpreter->LoadScript(m_szName);
#ifdef DEBUG_VERBOSE
		g_pfnMPrintf("+++ Instantiating PyScript %s [%p] on #%d\n", m_szName, m_pClass, m_iObjId);
#endif
	}
}

long cPyScript::ReceiveMessage(sScrMsg *pMsg, sMultiParm *pReply, eScrTraceAction eTrace)
{
	long iRet = 0;
	iRet = cScript::ReceiveMessage(pMsg, pReply, eTrace);
	if (!m_pClass)
		return -1;
	if (!stricmp(pMsg->message, "BeginScript"))
	{
		m_pScript = g_pScriptInterpreter->BeginScript(m_szName, m_pClass, m_iObjId);
	}
	if (!m_pScript)
	{
		if (!stricmp(pMsg->message, "Sim") && static_cast<sSimMsg*>(pMsg)->fStarting)
		{
			// Thief does wierd things with scripts on an AI.
			// It creates a new script instance after having sent the old instance 
			// "BeginScript". 
			// The new instance should get "Sim" start.
			m_pScript = g_pScriptInterpreter->BeginScript(m_szName, m_pClass, m_iObjId);
			if (!m_pScript)
				return -1;
			// Fake a BeginScript message.
			cMultiParm simReply;
			sScrMsg simMsg;
			simMsg.to = m_iObjId;
			simMsg.message = "BeginScript";
			g_pScriptInterpreter->ReceiveMessage(m_pScript, &simMsg, &simReply);
			simReply.Free();
		}
		else
		{
			return -1;
		}
	}
	iRet = g_pScriptInterpreter->ReceiveMessage(m_pScript, pMsg, pReply);
	if (!stricmp(pMsg->message, "EndScript"))
	{
		g_pScriptInterpreter->EndScript(m_pScript, m_pClass, m_iObjId);
		// We'll assume we're the only ones with a reference to the script. (Why wouldn't we be?)
		m_pScript = NULL;
	}
	return iRet;
}

IScript * PyScriptFactory(char *pszName, int iHostObjId)
{
	if (!g_pScriptInterpreter)
		return NULL;
	cPyScript *pScript = new(std::nothrow) cPyScript(pszName, iHostObjId);
	if (pScript)
		pScript->AddRef();
	return static_cast<IScript*>(pScript);
}

