/******************************************************************************
 *    Script.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#include <new>
#include <cstring>

#include <lg/scrmsgs.h>

#include "Script.h"


using namespace std;


cScript::~cScript()
{
	if (m_szName)
		delete[] m_szName;
}

cScript::cScript(const char* pszName, int iHostObjId)
	    : m_iRef(0), m_iObjId(iHostObjId)
{
	m_szName = new(nothrow) char[strlen(pszName)+1];
	if (m_szName)
		strcpy(m_szName, pszName);
}

HRESULT cScript::QueryInterface(REFIID riid, void** ppout)
{
	if (riid == IID_IUnknown)
		*ppout = this;
	else if (riid == IID_IScript)
		*ppout = static_cast<IScript*>(this);
	else
		return E_NOINTERFACE;
	static_cast<IUnknown*>(*ppout)->AddRef();
	return S_OK;
}

ULONG cScript::AddRef(void)
{
	return ++m_iRef;
}

ULONG cScript::Release(void)
{
	if (m_iRef)
	{
		if (--m_iRef == 0)
			delete this;
	}
	return m_iRef;
}

const char* cScript::GetClassName(void)
{
	// Name MUST match the one in the list.
	// Still, we don't want to toss a NULL pointer around, do we?
	return (m_szName) ? m_szName : "cScript";
}

long cScript::ReceiveMessage(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	long iRet = 0;
	if (!stricmp(pMsg->message, "ScriptPtrQuery"))
	{
		iRet = ScriptPtrQuery(pMsg, pReply, eTrace);
	}
	return iRet;
}


long cScript::ScriptPtrQuery(sScrMsg* pMsg, sMultiParm* pReply, eScrTraceAction eTrace)
{
	sPtrQueryMsg* pPtrQ = static_cast<sPtrQueryMsg*>(pMsg);
	// Check class name 
	if (!stricmp(pPtrQ->pszDestClass, GetClassName()))
	{
		*(pPtrQ->pScriptReceptacle) = GetThisPointer();
		return 0;
	}
	return 1;
}


