/******************************************************************************
 *    ScriptServices.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/defs.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"

namespace PyNumbra
{

	template<class _IFace>
	ScriptServiceObject<_IFace> * 
	ScriptService<_IFace>::NewObject(PyObject *)
	{
		_IFace *iface = static_cast<_IFace*>(g_pScriptManager->GetService(IID_INullScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<_IFace> *obj = PyObject_New(ScriptServiceObject<_IFace>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	template<class _IFace>
	void 
	ScriptService<_IFace>::Dealloc(ScriptServiceObject<_IFace> *self)
	{
		reinterpret_cast<_IFace*>(self->m_pIFace)->Release();
		self->ob_type->tp_free(static_cast<PyObject*>(self));
	}

	template<class _IFace>
	PyObject * 
	ScriptService<_IFace>::Repr(ScriptServiceObject<_IFace> *self)
	{
		return PyString_FromFormat("<PyNumbra ScriptService interface '%s'>", self->ob_type->tp_name);
	}

	template<class _IFace>
	int 
	ScriptService<_IFace>::Print(ScriptServiceObject<_IFace> *self, FILE *fp, int )
	{
		fprintf(fp, "<PyNumbra ScriptService interface '%s'>", self->ob_type->tp_name);
		return 0;
	}

	template<class _IFace>
	int 
	ScriptService<_IFace>::Compare(ScriptServiceObject<_IFace> *self, ScriptServiceObject<_IFace> *other)
	{
		return strcmp(self->ob_type->tp_name, other->ob_type->tp_name);
	}

	template<class _IFace>
	const char *
	ScriptService<_IFace>::Name = "ScriptService";

	template<class _IFace>
	const char * 
	ScriptService<_IFace>::DocStrings[] = { "" };

	template<class _IFace>
	const PyMethodDef  
	ScriptService<_IFace>::Methods[] = {
		{ NULL, NULL, 0, NULL }
	};

	template<class _IFace>
	PyTypeObject 
	ScriptService<_IFace>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(Name),
		sizeof(ScriptServiceObject<_IFace>), 0,
		reinterpret_cast<destructor>(Dealloc),
		reinterpret_cast<printfunc>(Print),		/* tp_print */
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(Compare),	/* tp_compare */
		reinterpret_cast<reprfunc>(Repr),	/* tp_repr */
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		reinterpret_cast<reprfunc>(Repr),	/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(DocStrings[0]),	/* tp_doc */
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(Methods),		/* tp_methods */
		0, 0, 0, 0, 0, 0, 0,
		0, 0, 0,	/* tp_init, tp_alloc, tp_new */
		0, 0, 0, 0, 0, 0, 0, 0
	};


// ScriptService<IUnknown> (ensures the template is instantiated)
	PyTypeObject 
	ScriptService<IUnknown>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IUnknown>::Name),
		sizeof(ScriptServiceObject<IUnknown>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		reinterpret_cast<printfunc>(ScriptService<IUnknown>::Print),
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IUnknown>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IUnknown>::Methods),
		0,0,0,0,0,0,0,0,0,0,0,0,
		0,0,0,0,0,0
	};


	ScriptServiceObject<IUnknown> * 
	ScriptService<IUnknown>::NewObject(PyObject *)
	{
		IUnknown *iface = static_cast<IUnknown*>(g_pScriptManager->GetService(IID_INullScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IUnknown> *obj = PyObject_New(ScriptServiceObject<IUnknown>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}


// ScriptServices Module
	static PyMethodDef ScriptServicesMethods[] = {
		{	const_cast<char*>(ScriptService<IActReactSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IActReactSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IActReactSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IAIScrSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IAIScrSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IAIScrSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IAnimTextureSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IAnimTextureSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IAnimTextureSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IBowSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IBowSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IBowSrv>::DocStrings[0])
		},
#if (_DARKGAME == 2)
		{	const_cast<char*>(ScriptService<ICameraSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<ICameraSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<ICameraSrv>::DocStrings[0])
		},
#endif
		{	const_cast<char*>(ScriptService<IContainSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IContainSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IContainSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IDamageSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IDamageSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IDamageSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IDarkGameSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IDarkGameSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IDarkGameSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IDarkUISrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IDarkUISrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IDarkUISrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IDataSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IDataSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IDataSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IDebugScrSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IDebugScrSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IDebugScrSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IDoorSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IDoorSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IDoorSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IDarkInvSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IDarkInvSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IDarkInvSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IDarkPowerupsSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IDarkPowerupsSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IDarkPowerupsSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IKeySrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IKeySrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IKeySrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<ILightScrSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<ILightScrSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<ILightScrSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<ILinkSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<ILinkSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<ILinkSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<ILinkToolsSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<ILinkToolsSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<ILinkToolsSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<ILockSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<ILockSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<ILockSrv>::DocStrings[0])
		},
#if (_DARKGAME == 3)
		{	const_cast<char*>(ScriptService<INetworkingSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<INetworkingSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<INetworkingSrv>::DocStrings[0])
		},
#endif
		{	const_cast<char*>(ScriptService<IObjectSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IObjectSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IObjectSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IPGroupSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IPGroupSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IPGroupSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IPhysSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IPhysSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IPhysSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IPickLockSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IPickLockSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IPickLockSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IPlayerLimbsSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IPlayerLimbsSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IPlayerLimbsSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IPropertySrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IPropertySrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IPropertySrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IPuppetSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IPuppetSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IPuppetSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IQuestSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IQuestSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IQuestSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<ISoundScrSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<ISoundScrSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<ISoundScrSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IWeaponSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IWeaponSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IWeaponSrv>::DocStrings[0])
		},
#if (_DARKGAME == 3)
		{	const_cast<char*>(ScriptService<IShockGameSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IShockGameSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IShockGameSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IShockObjSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IShockObjSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IShockObjSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IShockPsiSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IShockPsiSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IShockPsiSrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IShockAISrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IShockAISrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IShockAISrv>::DocStrings[0])
		},
		{	const_cast<char*>(ScriptService<IShockWeaponSrv>::Name), 
			reinterpret_cast<PyCFunction>(ScriptService<IShockWeaponSrv>::NewObject), 
			METH_NOARGS, 
			const_cast<char*>(ScriptService<IShockWeaponSrv>::DocStrings[0])
		},
#endif
		{ NULL, NULL, 0, NULL }
	};

	static const char *ScriptServicesDocString = "PyNumbra Script Services";

	extern "C" PyObject * 
	initScriptServices(void)
	{
		PyType_Ready(&ScriptService<IUnknown>::TypeInfo);

		PyObject *module = Py_InitModule3("PyNumbra.ScriptServices", ScriptServicesMethods, const_cast<char*>(ScriptServicesDocString));
		if (module)
		{
			ScriptService<IActReactSrv>::InitModule(module);
			ScriptService<IAIScrSrv>::InitModule(module);
			ScriptService<IAnimTextureSrv>::InitModule(module);
			ScriptService<IBowSrv>::InitModule(module);
#if (_DARKGAME == 2)
			ScriptService<ICameraSrv>::InitModule(module);
#endif
			ScriptService<IContainSrv>::InitModule(module);
			ScriptService<IDamageSrv>::InitModule(module);
			ScriptService<IDarkGameSrv>::InitModule(module);
			ScriptService<IDarkUISrv>::InitModule(module);
			ScriptService<IDataSrv>::InitModule(module);
			ScriptService<IDebugScrSrv>::InitModule(module);
			ScriptService<IDoorSrv>::InitModule(module);
			ScriptService<IDarkInvSrv>::InitModule(module);
			ScriptService<IDarkPowerupsSrv>::InitModule(module);
			ScriptService<IKeySrv>::InitModule(module);
			ScriptService<ILightScrSrv>::InitModule(module);
			ScriptService<ILinkSrv>::InitModule(module);
			ScriptService<ILinkToolsSrv>::InitModule(module);
			ScriptService<ILockSrv>::InitModule(module);
#if (_DARKGAME == 3)
			ScriptService<INetworkingSrv>::InitModule(module);
#endif
			ScriptService<IObjectSrv>::InitModule(module);
			ScriptService<IPGroupSrv>::InitModule(module);
			ScriptService<IPhysSrv>::InitModule(module);
			ScriptService<IPickLockSrv>::InitModule(module);
			ScriptService<IPlayerLimbsSrv>::InitModule(module);
			ScriptService<IPropertySrv>::InitModule(module);
			ScriptService<IPuppetSrv>::InitModule(module);
			ScriptService<IQuestSrv>::InitModule(module);
			ScriptService<ISoundScrSrv>::InitModule(module);
			ScriptService<IWeaponSrv>::InitModule(module);
#if (_DARKGAME == 3)
			ScriptService<IShockGameSrv>::InitModule(module);
			ScriptService<IShockObjSrv>::InitModule(module);
			ScriptService<IShockPsiSrv>::InitModule(module);
			ScriptService<IShockAISrv>::InitModule(module);
			ScriptService<IShockWeaponSrv>::InitModule(module);
#endif
		}
		return module;
	}

} // namespace PyNumbra

