/******************************************************************************
 *    ShockPsiService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#if (_DARKGAME == 3)
#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IShockPsiSrv>::Name = "ShockPsiService";

	const char * 
	ScriptService<IShockPsiSrv>::DocStrings[] = {
		"ShockPsiService()\n\t"
			"Only available in System Shock 2",
		"OnDeactivate(psi) -> Integer",
		"GetActiveTime(psi) -> Integer",
		"IsOverloaded(psi) -> Boolean",
	};

	const PyMethodDef 
	ScriptService<IShockPsiSrv>::Methods[] = {
		{	"OnDeactivate",
			reinterpret_cast<PyCFunction>(OnDeactivate),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"GetActiveTime",
			reinterpret_cast<PyCFunction>(GetActiveTime),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"IsOverloaded",
			reinterpret_cast<PyCFunction>(IsOverloaded),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IShockPsiSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IShockPsiSrv>::Name),
		sizeof(ScriptServiceObject<IShockPsiSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IShockPsiSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IShockPsiSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	void 
	ScriptService<IShockPsiSrv>::InitModule(PyObject *module)
	{
		PyType_Ready(&TypeInfo);
		// XXX: ePsiPowers constants
	}

	ScriptServiceObject<IShockPsiSrv> * 
	ScriptService<IShockPsiSrv>::NewObject(PyObject *)
	{
		IShockPsiSrv *iface = static_cast<IShockPsiSrv*>(g_pScriptManager->GetService(IID_IShockPsiScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IShockPsiSrv> *obj = PyObject_New(ScriptServiceObject<IShockPsiSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IShockPsiSrv>::OnDeactivate(ScriptServiceObject<IShockPsiSrv> *self, PyObject *args)
	{
		int psi;
		if (!PyArg_ParseTuple(args, "i:OnDeactivate", &psi))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->OnDeactivate(psi));
	}

	PyObject * 
	ScriptService<IShockPsiSrv>::GetActiveTime(ScriptServiceObject<IShockPsiSrv> *self, PyObject *args)
	{
		int psi;
		if (!PyArg_ParseTuple(args, "i:GetActiveTime", &psi))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->GetActiveTime(psi));
	}

	PyObject * 
	ScriptService<IShockPsiSrv>::IsOverloaded(ScriptServiceObject<IShockPsiSrv> *self, PyObject *args)
	{
		int psi;
		if (!PyArg_ParseTuple(args, "i:IsOverloaded", &psi))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->IsOverloaded(psi)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}


} // namespace PyNumbra

#endif
