/******************************************************************************
 *    pyutils.h
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#ifndef PYNUMBRA_UTILS_H
#define PYNUMBRA_UTILS_H

#include <Python.h>
#include <lg/types.h>

extern int (__cdecl *g_pfnMPrintf)(const char*,...);

#ifdef DEBUG_VERBOSE
#define DEBUGPRINTF g_pfnMPrintf
#else
#define DEBUGPRINTF(...)	
#endif

namespace PyNumbra
{

char* strsep(char** str, const char* delim);
unsigned int strtokcount(const char *str, const char *delim);

PyObject *GetObjectName(PyObject *ob);

void monoprintf(const char* pszFormat, ...);
void MonoOutput(PyObject *ob, bool status=false);

// This is reference-neutral
const char * GetInternString(PyObject *str);

// Conversion functions for passing to PyArg_ParseTuple
int cvtObject(PyObject *ob, object *val);
int cvtLink(PyObject *ob, link *val);
// If the conversion is successful, mp will not be kMT_Undef
int cvtMultiParm(PyObject *ob, cMultiParm *mp);

int MultiParmFromObject(cMultiParm &mp, PyObject *ob);
PyObject * MultiParmToObject(const cMultiParm& mp);

// Converts data with a StructDesc into a dictionary, or simple type
// ... hopefully. This function needs some serious mojo.
PyObject * StructDataToObject(const char *pszName, const char *pData);
// more pipe-dreams
//unsigned long StructDataFromObject(const char *pszName, PyObject *obData, char *pData);

} // namespace PyNumbra

#endif // PYNUMBRA_UTILS_H
